#version 430 core

layout (std140, binding = 0) uniform Common
{
	mat4 view;
	mat4 projection;
	mat4 viewProjection;
	vec3 cameraPos;
	int useDepthTexture;
};

struct PointLight
{
	vec3 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	vec3 attenuation;
	int enabled;
};

in vec3 FragPos;
in vec3 Normal;
in vec2 TextureCoord;
out vec4 FragColor;

uniform vec2 screenSize;
uniform sampler2D texture_diffuse1;
uniform float texturePercent;
uniform sampler2D depthTexture;

uniform PointLight pointLight;

vec3 calculateBasicLight(vec3 ambientColor, vec3 diffuseColor, vec3 specularColor, vec3 normal, vec3 lightDirection, vec3 viewDirection)
{
	// ambient
	vec3 ambient = ambientColor;    
	// diffuse 
	float diff = max(dot(normal, -lightDirection), 0.0);
	vec3 diffuse = diff * diffuseColor;
	// specular
	vec3 reflectDirection = reflect(lightDirection, normal);  
	float spec = pow(max(dot(viewDirection, reflectDirection), 0.0), 32.0);
	vec3 specular = spec * specularColor;
	return (ambient + diffuse + specular) * vec3(texture(texture_diffuse1, TextureCoord));
}

vec3 calculatePointLight(PointLight pLight)
{
	vec3 lightDir = FragPos - pLight.position;
	float dist = length(lightDir);
	float attenuation = 1.0 / (pLight.attenuation.x + pLight.attenuation.y * dist + pLight.attenuation.z * (dist * dist));
	lightDir = normalize(lightDir);
	vec3 viewDir = normalize(cameraPos - FragPos);
	return calculateBasicLight(pLight.ambient, pLight.diffuse, pLight.specular, normalize(Normal), lightDir, viewDir) * attenuation;
}

void main()
{
	if (useDepthTexture == 1)
	{
		float depth = texture(depthTexture, gl_FragCoord.xy / screenSize).x;
		if (depth < gl_FragCoord.z)
		{
			FragColor = vec4(calculatePointLight(pointLight), 1.0);
		}
		else
		{
			discard;
		}
	}
	else
	{
		FragColor = vec4(calculatePointLight(pointLight), 1.0);
	}
}