#version 430 core

layout (std140, binding = 0) uniform Common
{
	mat4 view;
	mat4 projection;
	mat4 viewProjection;
	vec3 cameraPos;
	int useDepthTexture;
};

struct PointLight
{
	vec3 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	vec3 attenuation;
	int enabled;
};

in vec3 FragPos;
in mat3 TBN;
in vec2 TextureCoord;

out vec4 FragColor;

uniform vec2 screenSize;
uniform sampler2D depthTexture;

uniform sampler2D texture_diffuse1;
uniform sampler2D texture_normal1;
uniform sampler2D texture_height1;
uniform PointLight pointLight;

vec2 ParallaxMapping(vec2 texCoords, vec3 viewDir)
{ 
	float heightScale = 0.05;
    float height =  texture(texture_height1, texCoords).r;     
    return texCoords - viewDir.xy * (height * heightScale);        
}

vec3 calculateBasicLight(vec3 ambientColor, vec3 diffuseColor, vec3 specularColor, 
	vec3 normal, vec3 lightDirection, vec3 viewDirection, vec2 texCoords_offset)
{
	// ambient
	vec3 ambient = ambientColor;  
	// diffuse 
	float diff = max(dot(normal, lightDirection), 0.0);
	vec3 diffuse = diff * diffuseColor;
	// specular
	vec3 reflectDirection = reflect(-lightDirection, normal);  
	float spec = pow(max(dot(viewDirection, reflectDirection), 0.0), 32.0);
	vec3 specular = spec * specularColor;
	return (ambient + diffuse + specular) * vec3(texture(texture_diffuse1, texCoords_offset));
}

vec3 calculatePointLight(vec3 normal, vec3 viewDir, vec2 texCoords_offset, vec3 TangentLightPos, vec3 TangentFragPos)
{
	vec3 lightDir = TangentLightPos - TangentFragPos;
	float dist = length(lightDir);
	float attenuation = 1.0 / (pointLight.attenuation.x + pointLight.attenuation.y * dist + pointLight.attenuation.z * (dist * dist));
	lightDir = normalize(lightDir);
	return calculateBasicLight(pointLight.ambient, pointLight.diffuse, pointLight.specular, normal, lightDir, viewDir, texCoords_offset) * attenuation;
}

vec3 calculateFragment()
{
	vec3 TangentLightPos = TBN * pointLight.position;
    vec3 TangentViewPos = TBN * cameraPos;
    vec3 TangentFragPos = TBN * FragPos;

	// offset texture coordinates with Parallax Mapping
    vec3 viewDir = normalize(TangentViewPos - TangentFragPos);
	vec2 texCoords_offset = TextureCoord;
		
	texCoords_offset = ParallaxMapping(TextureCoord,  viewDir);

	// obtain normal from normal map
	vec3 normal = texture(texture_normal1, texCoords_offset).rgb;
	normal = normalize(normal * 2.0 - 1.0);

	return calculatePointLight(normal, viewDir, texCoords_offset, TangentLightPos, TangentFragPos);
}


void main()
{
	if (useDepthTexture == 1)
	{
		float depth = texture(depthTexture, gl_FragCoord.xy / screenSize).x;
		if (depth < gl_FragCoord.z)
		{
			FragColor = vec4(calculateFragment(), 1.0);
		}
		else
		{
			discard;
		}
	}
	else
	{
		FragColor = vec4(calculateFragment(), 1.0);	
	}
}