#version 430 core
layout (location = 0) in vec3 pos;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 textureCoord;
layout (location = 3) in ivec3 jointIndices;
layout (location = 4) in vec3 weights;
layout (location = 5) in vec3 tangent;

layout (std140, binding = 0) uniform Common
{
	mat4 view;
	mat4 projection;
	mat4 viewProjection;
	vec3 cameraPos;
	int useDepthTexture;
};

out vec3 FragPos;
out vec2 TextureCoord;
out mat3 TBN;

uniform mat4 model;
uniform mat3 normalMatrix;

void main()
{
    gl_Position = viewProjection * model * vec4(pos, 1.0);
    FragPos = vec3(model * vec4(pos, 1.0));
	TextureCoord = textureCoord;

	vec3 T = normalize(vec3(model * vec4(tangent, 0.0)));
	vec3 N = normalize(vec3(model * vec4(normal, 0.0)));
	// re-orthogonalize T with respect to N
	T = normalize(T - dot(T, N) * N);
	// then retrieve perpendicular vector B with the cross product of T and N
	vec3 B = cross(N, T);

	TBN = mat3(T, B, N);
}
