#version 430 core

#define MAX_JOINTS 30
#define MAX_WEIGHTS 3

layout (location = 0) in vec3 pos;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 textureCoord;
layout (location = 3) in ivec3 jointIndices;
layout (location = 4) in vec3 weights;

layout (std140, binding = 0) uniform Common
{
	mat4 view;
	mat4 projection;
	mat4 viewProjection;
	vec3 cameraPos;
	int useDepthTexture;
};

out vec3 FragPos;
out vec3 Normal;
out vec2 TextureCoord;

uniform mat4 model;
uniform mat3 normalMatrix;
uniform mat4 jointTransforms[MAX_JOINTS];

void main()
{
	vec4 totalLocalPos = vec4(0.0);
	vec4 totalNormal = vec4(0.0);

	for (int i = 0; i < MAX_WEIGHTS; i++)
	{
		mat4 jointTransform = jointTransforms[jointIndices[i]];
		totalLocalPos += jointTransform * vec4(pos, 1.0) * weights[i];
		totalNormal += jointTransform * vec4(normal, 0.0) * weights[i];
	}

    gl_Position = viewProjection * model * totalLocalPos;
    FragPos = vec3(model * totalLocalPos);
    Normal = normalMatrix * totalNormal.xyz;
	TextureCoord = textureCoord;
}
